import { useState, useEffect } from "react";
import { WeeklyReport, UpdateWeeklyReport } from "../Types/goTypes";
import { api } from "../API/API";
import { useNavigate, useParams } from "react-router-dom";
import Button from "./Button";

/**
 * Renders the component for editing a weekly report.
 * @returns JSX.Element
 */
export default function GetWeeklyReport(): JSX.Element {
  const [week, setWeek] = useState(0);
  const [developmentTime, setDevelopmentTime] = useState(0);
  const [meetingTime, setMeetingTime] = useState(0);
  const [adminTime, setAdminTime] = useState(0);
  const [ownWorkTime, setOwnWorkTime] = useState(0);
  const [studyTime, setStudyTime] = useState(0);
  const [testingTime, setTestingTime] = useState(0);

  const token = localStorage.getItem("accessToken") ?? "";
  const { projectName, fetchedWeek, signedOrUnsigned } = useParams<{
    projectName: string;
    fetchedWeek: string;
    signedOrUnsigned: string;
  }>();
  const username = localStorage.getItem("userName") ?? "";
  console.log(projectName, fetchedWeek, signedOrUnsigned);

  useEffect(() => {
    const fetchWeeklyReport = async (): Promise<void> => {
      const response = await api.getWeeklyReport(
        projectName ?? "",
        fetchedWeek ?? "",
        token,
      );

      if (response.success) {
        const report: WeeklyReport = response.data ?? {
          reportId: 0,
          userId: 0,
          projectId: 0,
          week: 0,
          developmentTime: 0,
          meetingTime: 0,
          adminTime: 0,
          ownWorkTime: 0,
          studyTime: 0,
          testingTime: 0,
        };
        setWeek(report.week);
        setDevelopmentTime(report.developmentTime);
        setMeetingTime(report.meetingTime);
        setAdminTime(report.adminTime);
        setOwnWorkTime(report.ownWorkTime);
        setStudyTime(report.studyTime);
        setTestingTime(report.testingTime);
      } else {
        console.error("Failed to fetch weekly report:", response.message);
      }
    };

    void fetchWeeklyReport();
  }, [projectName, fetchedWeek, signedOrUnsigned, token]);

  const handleUpdateWeeklyReport = async (): Promise<void> => {
    const updateWeeklyReport: UpdateWeeklyReport = {
      userName: username,
      projectName: projectName ?? "",
      week,
      developmentTime,
      meetingTime,
      adminTime,
      ownWorkTime,
      studyTime,
      testingTime,
    };

    await api.updateWeeklyReport(updateWeeklyReport, token);
  };

  const navigate = useNavigate();

  return (
    <>
      <h1 className="font-bold text-[30px] mb-[20px]">Edit Time Report</h1>
      <div className="border-4 border-black bg-white flex flex-col justify-start min-h-[65vh] h-fit w-[50vw] rounded-3xl overflow-scroll space-y-[2vh] p-[30px] items-center">
        <form
          onSubmit={(e) => {
            if (week === 0) {
              alert("Please enter a week number");
              e.preventDefault();
              return;
            }
            e.preventDefault();
            void handleUpdateWeeklyReport();
            alert("Changes submitted");
            navigate(-1);
          }}
        >
          <div className="flex flex-col items-center">
            <div className="flex flex-col w-1/2 border-b-2 border-black items-center justify-center">
              <h1 className="font-bold text-[30px]"> Week: {week}</h1>
            </div>

            <table className="w-full text-center divide-y divide-x divide-white text-[30px]">
              <thead>
                <tr>
                  <th className="w-1/2 py-2 border-b-2 border-black">
                    Activity
                  </th>
                  <th className="w-1/2 py-2 border-b-2 border-black">
                    Total Time (min)
                  </th>
                </tr>
              </thead>
              <tbody className="divide-y divide-black">
                <tr className="h-[10vh]">
                  <td>Development</td>
                  <td>
                    <input
                      type="number"
                      min="0"
                      className="border-2 border-black rounded-md text-center w-1/2"
                      value={developmentTime === 0 ? "" : developmentTime}
                      onChange={(e) => {
                        if (e.target.value === "") {
                          setDevelopmentTime(0);
                          return;
                        } else {
                          setDevelopmentTime(parseInt(e.target.value));
                        }
                      }}
                      onKeyDown={(event) => {
                        const keyValue = event.key;
                        if (
                          !/\d/.test(keyValue) &&
                          keyValue !== "Backspace" &&
                          keyValue !== "ArrowLeft" &&
                          keyValue !== "ArrowRight"
                        )
                          event.preventDefault();
                      }}
                      onClick={() => {
                        if (signedOrUnsigned === "signed") {
                          alert("You cannot edit a signed report.");
                        }
                      }}
                      readOnly={signedOrUnsigned === "signed"}
                    />
                  </td>
                </tr>
                <tr className="h-[10vh]">
                  <td>Meeting</td>
                  <td>
                    <input
                      type="number"
                      min="0"
                      className="border-2 border-black rounded-md text-center w-1/2"
                      value={meetingTime === 0 ? "" : meetingTime}
                      onChange={(e) => {
                        if (e.target.value === "") {
                          setMeetingTime(0);
                          return;
                        } else {
                          setMeetingTime(parseInt(e.target.value));
                        }
                      }}
                      onKeyDown={(event) => {
                        const keyValue = event.key;
                        if (
                          !/\d/.test(keyValue) &&
                          keyValue !== "Backspace" &&
                          keyValue !== "ArrowLeft" &&
                          keyValue !== "ArrowRight"
                        )
                          event.preventDefault();
                      }}
                      onClick={() => {
                        if (signedOrUnsigned === "signed") {
                          alert("You cannot edit a signed report.");
                        }
                      }}
                      readOnly={signedOrUnsigned === "signed"}
                    />
                  </td>
                </tr>
                <tr className="h-[10vh]">
                  <td>Administration</td>
                  <td>
                    <input
                      type="number"
                      min="0"
                      className="border-2 border-black rounded-md text-center w-1/2"
                      value={adminTime === 0 ? "" : adminTime}
                      onChange={(e) => {
                        if (e.target.value === "") {
                          setAdminTime(0);
                          return;
                        } else {
                          setAdminTime(parseInt(e.target.value));
                        }
                      }}
                      onKeyDown={(event) => {
                        const keyValue = event.key;
                        if (
                          !/\d/.test(keyValue) &&
                          keyValue !== "Backspace" &&
                          keyValue !== "ArrowLeft" &&
                          keyValue !== "ArrowRight"
                        )
                          event.preventDefault();
                      }}
                      onClick={() => {
                        if (signedOrUnsigned === "signed") {
                          alert("You cannot edit a signed report.");
                        }
                      }}
                      readOnly={signedOrUnsigned === "signed"}
                    />
                  </td>
                </tr>
                <tr className="h-[10vh]">
                  <td>Own Work</td>
                  <td>
                    <input
                      type="number"
                      min="0"
                      className="border-2 border-black rounded-md text-center w-1/2"
                      value={ownWorkTime === 0 ? "" : ownWorkTime}
                      onChange={(e) => {
                        if (e.target.value === "") {
                          setOwnWorkTime(0);
                          return;
                        } else {
                          setOwnWorkTime(parseInt(e.target.value));
                        }
                      }}
                      onKeyDown={(event) => {
                        const keyValue = event.key;
                        if (
                          !/\d/.test(keyValue) &&
                          keyValue !== "Backspace" &&
                          keyValue !== "ArrowLeft" &&
                          keyValue !== "ArrowRight"
                        )
                          event.preventDefault();
                      }}
                      onClick={() => {
                        if (signedOrUnsigned === "signed") {
                          alert("You cannot edit a signed report.");
                        }
                      }}
                      readOnly={signedOrUnsigned === "signed"}
                    />
                  </td>
                </tr>
                <tr className="h-[10vh]">
                  <td>Studies</td>
                  <td>
                    <input
                      type="number"
                      min="0"
                      className="border-2 border-black rounded-md text-center w-1/2"
                      value={studyTime === 0 ? "" : studyTime}
                      onChange={(e) => {
                        if (e.target.value === "") {
                          setStudyTime(0);
                          return;
                        } else {
                          setStudyTime(parseInt(e.target.value));
                        }
                      }}
                      onKeyDown={(event) => {
                        const keyValue = event.key;
                        if (
                          !/\d/.test(keyValue) &&
                          keyValue !== "Backspace" &&
                          keyValue !== "ArrowLeft" &&
                          keyValue !== "ArrowRight"
                        )
                          event.preventDefault();
                      }}
                      onClick={() => {
                        if (signedOrUnsigned === "signed") {
                          alert("You cannot edit a signed report.");
                        }
                      }}
                      readOnly={signedOrUnsigned === "signed"}
                    />
                  </td>
                </tr>
                <tr className="h-[10vh]">
                  <td>Testing</td>
                  <td>
                    <input
                      type="number"
                      min="0"
                      className="border-2 border-black rounded-md text-center w-1/2"
                      value={testingTime === 0 ? "" : testingTime}
                      onChange={(e) => {
                        if (e.target.value === "") {
                          setTestingTime(0);
                          return;
                        } else {
                          setTestingTime(parseInt(e.target.value));
                        }
                      }}
                      onKeyDown={(event) => {
                        const keyValue = event.key;
                        if (
                          !/\d/.test(keyValue) &&
                          keyValue !== "Backspace" &&
                          keyValue !== "ArrowLeft" &&
                          keyValue !== "ArrowRight"
                        )
                          event.preventDefault();
                      }}
                      onClick={() => {
                        if (signedOrUnsigned === "signed") {
                          alert("You cannot edit a signed report.");
                        }
                      }}
                      readOnly={signedOrUnsigned === "signed"}
                    />
                  </td>
                </tr>
              </tbody>
            </table>
            {signedOrUnsigned !== "signed" && (
              <Button
                text="Submit changes"
                onClick={(): void => {
                  return;
                }}
                type="submit"
              />
            )}
          </div>
        </form>
      </div>
    </>
  );
}
